'use strict';
// This module handles the KML export.

/* global appCore */
function KMLExport() {
    var self = this;

    var DATA_KEY_CONFIG = 'imatch.mapapp.{u}.kml-export-config';
    var DATA_KEY_CONFIG_HELP = 'imatch.mapapp.{u}.kml-export-config-help';

    var config = {
        version: 1,
        fileName : '',
        thumbSize : 0,
        docTitle : '',
        docDesc: '',
        fileTitle: '',
        fileDesc: '',
    }

    var showHelp = true;

    self.open = function()
    {
        // The per-user data storage key
        DATA_KEY_CONFIG = DATA_KEY_CONFIG.replace('{u}',appCore.userName);
        DATA_KEY_CONFIG_HELP = DATA_KEY_CONFIG_HELP.replace('{u}',appCore.userName);

        appCore.setActiveOverlay('kml-export');

        $.get('kml/kml-export-tpl.html', function(data) {

            // Add template to DOM
            $('#app-container').append(data);

            var kf = $('#kml-form');

            // Translate the modal and all children
            IMatchTranslate.translateElement('#kml-export-container');

            // Fill in  the available thumbnail sizes
            var select = $('#kml-export-thumbsize');
            const TSIZES = [80,100,120,160,180,200,240,300];
            TSIZES.forEach(function(s) {
                var frm = IMatchTranslate.getStr('KML_EXPORT_THUMBNAILSIZEFORMAT');
                frm = frm.replace('{0}',s);
                select.append('<option value="' + s + '">' + frm + '</option>');
            }, this);

            $('#kml-export-container-close').click(self.close);
            $('#kml-export-btn-cancel').click(self.close);
            $('#kml-export-btn-ok').click(runExport);
            $('#kml-export-btn-filename-browse').click(browseFileName);
            $('#btn-toggle-help').click(toggleHelp);
            $('#kml-export-btn-result-open').click(openResultInWinowsExplorer);
            $('#kml-export-btn-result-close').click(self.close);

            
            IMWS.dataGet({name: DATA_KEY_CONFIG_HELP}).then(function(response) {
                // We load the help state first.
                var data = JSON.parse(response.value);
                showHelp = data;
            }).always(function() {
                // Load the configuration data from the last session
                // if available. Then update the form and show the overlay.
                IMWS.dataGet({name: DATA_KEY_CONFIG}).then(function(response) {
                    var data = JSON.parse(response.value);
                    // Check the version
                    if (data.version == config.version) {
                        config = data;
                    }
                },function(error) {
                    // Settings not found.
                    setupDefaults();
                }).always(function() {
                    // Finally, update the form and show the overlay
                    updateForm();
                    setHelpVisibility(false);
                    $('#kml-export-container').css('width','100%');
                    $('#kml-export-container').css('height','100%');
                });
            });
        });

        $(document).keydown(function(event) {
            if (event.which === 27) { // ESC
                self.close();
            }
        });        
    }

    self.close = function()
    {
        $('#kml-export-container').css('width','0%');
        $('#kml-export-container').css('height','0%');
        $('#kml-export-container').remove();
        
        appCore.setActiveOverlay('');
    }

   

    ///////////////////////////////////////////////////////////////////////////
    // Run the export
    function runExport(event)
    {
        updateConfig();

        // We hide the form and show the export div instead
        $('#kml-form-container').hide(function() {

            $('#kml-export-run-container').show();
            $('#kml-export-run-container-running').show();

            IMWS.post('v1/utility/createkmzfile',{
                idlist: IMatch.idlist.fileWindowSelection,
                kmzfile: config.fileName,
                thumbsize: config.thumbSize,
                documenttitle: config.docTitle,
                documentdescription: config.docDesc,
                filetitle: config.fileTitle,
                filedescription: config.fileDesc,
            }).then(function(response) {
                // We save the configuration when the user performs an export.
                config.version = 1;
                IMWS.dataSet({
                    name: DATA_KEY_CONFIG, 
                    value: JSON.stringify(config)
                });

                // Done
                $('#kml-export-run-container-running').hide();
                $('#kml-export-run-container-success').fadeIn();
            },function(error){
                // Oops
                $('#kml-export-run-container-running').hide();
                $('#kml-export-run-container-failure').fadeIn();
                $('#kml-export-run-container-failure-text').text(JSON.stringify(error.responseJSON,null,2));
            })
        });
        
        event.preventDefault();
        return false;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Display a file browser dialog
    function browseFileName(event)
    {
        updateConfig();

        IMatch.fileBrowserFS({
            type: 'open',
            initialdir: config.fileName,
            filename: config.fileName,
            defext: '.kmz',
            title: IMatchTranslate.getStr('KML_EXPORT_FILEBROWSER_TITLE'),
            filter: IMatchTranslate.getStr('KML_EXPORT_FILEBROWSER_FILTER'),
        }).then(function(response) {
            if (response.result == 'ok') {
                config.fileName = response.fileName;
                updateForm();
            }
        });

        event.preventDefault();
        return false;
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    function openResultInWinowsExplorer()
    {
        IMatch.shellExecute({
            verb: 'open',
            executable: 'explorer.exe',
            parameters: '/select,"' + config.fileName + '"'
        });
    }

    ///////////////////////////////////////////////////////////////////////////
    // Setup default values for the config
    function setupDefaults()
    {
        // We access the special folders loaded by the app
        config.fileName = appCore.specialFolders.MYDOCUMENTS + 'imatch.kmz';
        
        // No thumbs
        config.thumbSize = 0;

        config.docTitle = IMatchTranslate.getStr('KML_EXPORT_DOCTITLE_DEFAULT');
        config.docDesc = IMatchTranslate.getStr('KML_EXPORT_DOCDESC_DEFAULT');
        config.fileTitle = IMatchTranslate.getStr('KML_EXPORT_FILETITLE_DEFAULT');
        config.fileDesc = IMatchTranslate.getStr('KML_EXPORT_FILEDESC_DEFAULT');
    }


    ///////////////////////////////////////////////////////////////////////////
    //
    function toggleHelp()
    {
        showHelp = !showHelp;
        setHelpVisibility(true);

        IMWS.dataSet({
            name: DATA_KEY_CONFIG_HELP, 
            value: JSON.stringify(showHelp)
        });
    }
   
    ///////////////////////////////////////////////////////////////////////////
    //
    function setHelpVisibility(animate)
    {
        if (showHelp) {
            $('.kml-export-form-help').fadeIn(animate === true ? 400 : 0);
        }
        else {
            $('.kml-export-form-help').fadeOut(animate === true ? 400 : 0);
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    // Update form from config data
    function updateForm()
    {
        $('#kml-export-filename').val(config.fileName);
        $('#kml-export-thumbsize option[value="' + config.thumbSize + '"]').prop('selected', true);
        $('#kml-export-doctitle').val(config.docTitle);
        $('#kml-export-docdesc').val(config.docDesc);
        $('#kml-export-filetitle').val(config.fileTitle);
        $('#kml-export-filedesc').val(config.fileDesc);
    }

    ///////////////////////////////////////////////////////////////////////////
    // Update config data from form
    function updateConfig()
    {
        config.fileName = $('#kml-export-filename').val();
        config.thumbSize = $('#kml-export-thumbsize').val();
        config.docTitle = $('#kml-export-doctitle').val();
        config.docDesc = $('#kml-export-docdesc').val();
        config.fileTitle = $('#kml-export-filetitle').val();
        config.fileDesc = $('#kml-export-filedesc').val();
    }
   
}
