'use strict';

// This module manages the dialogs etc. we need.

/* global appCore,IMatchTranslate */
function MapModals() {
    var self = this;

    var progressVisible = false;

    // Closes open modals or progress overlays.
    // No callbacks (e.g. for YesNo) will be called!
    self.closeAll = function()
    {
        self.closeProgressOverlay();
    }

    // Display a full-size overlay with a headline, text and progress bar.
    // createCallback is called once the overlay has been added to the DOM.
    // abortCallback is called when the user clicks on the abort button (only when showAbort is true)
    self.showProgressOverlay = function(showAbort,createCallback,abortCallback)
    {
        $.get('modals/modal-progress-tpl.html', function(data) {

            progressVisible = true;

            $('#imm-progress-pbar').css('width','0%');

            // Add template to DOM
            $('#app-container').append(data);

            // Translate the modal and all children
            IMatchTranslate.translateElement('#imm-modal');

            if (showAbort) {
                $('#imm-progress-abort').show();
                $('#imm-progress-abort').removeClass('disabled');
            }

            if (createCallback) {
                createCallback($('#imm-modal'));
            }

            // Open the modal
            $('#imm-modal').modal({});

            // Remove the HTML template when the modal is closed.
            // Call the onCancel callback of the modal was not closed by the OK button.
            $('#imm-modal').on('hidden.bs.modal', function (e) {
                var e = $('#imm-modal');
                $('#imm-modal').remove();
            });

            // Handle the OK button.
            // Close the modal and call the onOK callback.
            $('#imm-progress-abort').click(function (e) {
                if (showAbort) {
                    $('#imm-modal').modal('hide');
                    progressVisible = false;
                    if (abortCallback) abortCallback();
                }
            });
        });
    }

    // Set a new percentage between 0 and 100. 
    // Optional set a new title and/or text
    self.setProgressOverlayPercentage = function(perc,title,text)
    {
        if (progressVisible) {
            var p = Math.max(0,Math.min(100,perc));
            $('#imm-progress-pbar').css('width', p + '%');
            $('#imm-progress-pbar').text(Math.floor(p) + '%');

            if (title !== undefined) {
                $('#modal-headline').text(title);
            }
            if (text !== undefined) {
                $('#modal-content').text(text);
            }
       }
    }

    // Close the current progress overlay if there is one
    self.closeProgressOverlay = function()
    {
        if (progressVisible) {
            progressVisible = false;
            $('#imm-modal').modal('hide');
       }
    }
    
}